@@ -4,12 +4,14 @@ import yaml
 import logging
 from collections import deque
 import copy
+import base64
 
 from flask import Flask
 from flask import jsonify
 from flask import request
 from flask import abort
 from flask import Response
+from flask_cors import CORS
 
 import requests
 
@@ -75,6 +77,7 @@ SchemaVersion.migrate_pendings()
 SchemaVersion.set_version()
 
 app = Flask(__name__)
+CORS(app)
 
 biomaj_metric = Counter("biomaj_daemon_total", "Bank total update execution.", ['bank', 'action', 'updated'])
 biomaj_error_metric = Counter("biomaj_daemon_errors", "Bank total update errors.", ['bank', 'action'])
@@ -156,7 +159,8 @@ OPTIONS_PARAMS = {
     'history': False,
     'historyLimit': 20,
     'datalist': False,
-    'dataimport': False
+    'dataimport': False,
+    'repair': False
 }
 
 
@@ -678,7 +682,7 @@ def biomaj_daemon_bank_update_status(bank):
 
 
 @app.route('/api/daemon/status/ko', methods=['GET'])
-def biomaj_daemon_bank_status_ko(bank):
+def biomaj_daemon_bank_status_ko():
     (http_code, options, error) = daemon_api_auth(request)
     if error:
         abort(http_code, error)
@@ -795,6 +799,31 @@ def biomaj_daemon_bank_update_directory(bank):
         abort(500, str(e))
 
 
+@app.route('/api/daemon/bank/<bank>/repair', methods=['PUT'])
+def biomaj_daemon_bank_repair(bank):
+    (http_code, options, error) = daemon_api_auth(request)
+    if error:
+        abort(http_code, error)
+    if not options.user:
+        abort(401, 'This action requires authentication with api key')
+    options.repair = True
+    options.bank = bank
+    params = request.get_json()
+    if params is None:
+        params = {}
+    if 'release' in params:
+        options.release = params['release']
+    try:
+        (res, msg) = biomaj_client_action(options, config)
+        if res:
+            if isinstance(msg, dict):
+                return jsonify(msg)
+            else:
+                return jsonify({'msg': msg})
+    except Exception as e:
+        abort(500, str(e))
+
+
 @app.route('/api/daemon/bank/<bank>', methods=['POST'])
 def biomaj_daemon_bank_update(bank):
     (http_code, options, error) = daemon_api_auth(request)
@@ -1079,6 +1108,46 @@ def add_metrics():
     return jsonify({'msg': 'OK'})
 
 
+@app.route('/api/daemon/expose', methods=['GET'])
+def expose():
+    do_expose = True
+    try:
+        do_expose_cfg = BiomajConfig.global_config.get('GENERAL', 'expose')
+        if do_expose_cfg == "false" or do_expose_cfg == "0":
+            do_expose = False
+
+    except Exception:
+        do_expose = True
+    if not do_expose:
+        abort(403)
+    uri = request.headers['X-Forwarded-Uri']
+    if uri == '/db' or uri == '/db/':
+        return jsonify({'msg': 'access allowed'})
+    elts = uri.split('/')
+    requested_bank = elts[2]
+    user = None
+    options = Options({})
+    bank = Bank(requested_bank, options=Options({}), no_log=True)
+    if bank.bank['properties']['visibility'] == 'public':
+        return jsonify({'msg': 'access allowed'})
+
+    if not request.headers.get('Authorization', None):
+        abort(403)
+    encoded_uname_pass = request.headers.get('Authorization').split()[-1]
+    uname_pass = base64.b64decode(encoded_uname_pass)
+    apikey = uname_pass.split(':')[-1]
+    proxy = Utils.get_service_endpoint(config, 'user')
+    r = requests.get(proxy + '/api/user/info/apikey/' + apikey)
+    if not r.status_code == 200:
+        abort(404, {'message': 'Invalid API Key or connection issue'})
+    user = r.json()['user']
+    options = Options({'user': user['id']})
+    bank = Bank(requested_bank, options=options, no_log=True)
+    if not bank.is_owner():
+        abort(403)
+    return jsonify({'msg': 'access allowed'})
+
+
 if __name__ == "__main__":
     context = None
     if config['tls']['cert']:
